@@ -64,21 +64,21 @@ static check_tgs_svc_pol_fn * const svc_pol_fns[] = {
 
 static const struct tgsflagrule tgsflagrules[] = {
     { KDC_OPT_FORWARDED, TKT_FLG_FORWARDABLE,
-      "TGT NOT FORWARDABLE", KDC_ERR_BADOPTION },
+      "TGT NOT FORWARDABLE", KRB5KDC_ERR_BADOPTION },
     { KDC_OPT_PROXY, TKT_FLG_PROXIABLE,
-      "TGT NOT PROXIABLE", KDC_ERR_BADOPTION },
+      "TGT NOT PROXIABLE", KRB5KDC_ERR_BADOPTION },
     { (KDC_OPT_ALLOW_POSTDATE | KDC_OPT_POSTDATED), TKT_FLG_MAY_POSTDATE,
-      "TGT NOT POSTDATABLE", KDC_ERR_BADOPTION },
+      "TGT NOT POSTDATABLE", KRB5KDC_ERR_BADOPTION },
     { KDC_OPT_VALIDATE, TKT_FLG_INVALID,
-      "VALIDATE VALID TICKET", KDC_ERR_BADOPTION },
+      "VALIDATE VALID TICKET", KRB5KDC_ERR_BADOPTION },
     { KDC_OPT_RENEW, TKT_FLG_RENEWABLE,
-      "TICKET NOT RENEWABLE", KDC_ERR_BADOPTION }
+      "TICKET NOT RENEWABLE", KRB5KDC_ERR_BADOPTION }
 };
 
 /*
  * Some TGS-REQ options require that the ticket have corresponding flags set.
  */
-static int
+static krb5_error_code
 check_tgs_opts(krb5_kdc_req *req, krb5_ticket *tkt, const char **status)
 {
     size_t i;
@@ -98,7 +98,7 @@ check_tgs_opts(krb5_kdc_req *req, krb5_ticket *tkt, const char **status)
     if (isflagset(tkt->enc_part2->flags, TKT_FLG_INVALID) &&
         !isflagset(req->kdc_options, KDC_OPT_VALIDATE)) {
         *status = "TICKET NOT VALID";
-        return KRB_AP_ERR_TKT_NYV;
+        return KRB5KRB_AP_ERR_TKT_NYV;
     }
 
     return 0;
@@ -106,17 +106,17 @@ check_tgs_opts(krb5_kdc_req *req, krb5_ticket *tkt, const char **status)
 
 static const struct tgsflagrule svcdenyrules[] = {
     { KDC_OPT_RENEWABLE, KRB5_KDB_DISALLOW_RENEWABLE,
-      "NON-RENEWABLE TICKET", KDC_ERR_POLICY },
+      "NON-RENEWABLE TICKET", KRB5KDC_ERR_POLICY },
     { KDC_OPT_ALLOW_POSTDATE, KRB5_KDB_DISALLOW_POSTDATED,
-      "NON-POSTDATABLE TICKET", KDC_ERR_CANNOT_POSTDATE },
+      "NON-POSTDATABLE TICKET", KRB5KDC_ERR_CANNOT_POSTDATE },
     { KDC_OPT_ENC_TKT_IN_SKEY, KRB5_KDB_DISALLOW_DUP_SKEY,
-      "DUP_SKEY DISALLOWED", KDC_ERR_POLICY }
+      "DUP_SKEY DISALLOWED", KRB5KDC_ERR_POLICY }
 };
 
 /*
  * A service principal can forbid some TGS-REQ options.
  */
-static int
+static krb5_error_code
 check_tgs_svc_deny_opts(krb5_kdc_req *req, krb5_db_entry *server,
                         krb5_ticket *tkt, krb5_timestamp kdc_time,
                         const char **status)
@@ -140,24 +140,24 @@ check_tgs_svc_deny_opts(krb5_kdc_req *req, krb5_db_entry *server,
 /*
  * A service principal can deny all TGS-REQs for it.
  */
-static int
+static krb5_error_code
 check_tgs_svc_deny_all(krb5_kdc_req *req, krb5_db_entry *server,
                        krb5_ticket *tkt, krb5_timestamp kdc_time,
                        const char **status)
 {
     if (server->attributes & KRB5_KDB_DISALLOW_ALL_TIX) {
         *status = "SERVER LOCKED OUT";
-        return KDC_ERR_S_PRINCIPAL_UNKNOWN;
+        return KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN;
     }
     if ((server->attributes & KRB5_KDB_DISALLOW_SVR) &&
         !(req->kdc_options & KDC_OPT_ENC_TKT_IN_SKEY)) {
         *status = "SERVER NOT ALLOWED";
-        return KDC_ERR_MUST_USE_USER2USER;
+        return KRB5KDC_ERR_MUST_USE_USER2USER;
     }
     if (server->attributes & KRB5_KDB_DISALLOW_TGT_BASED) {
         if (krb5_is_tgs_principal(tkt->server)) {
             *status = "TGT BASED NOT ALLOWED";
-            return KDC_ERR_POLICY;
+            return KRB5KDC_ERR_POLICY;
         }
     }
     return 0;
@@ -166,7 +166,7 @@ check_tgs_svc_deny_all(krb5_kdc_req *req, krb5_db_entry *server,
 /*
  * A service principal can require certain TGT flags.
  */
-static int
+static krb5_error_code
 check_tgs_svc_reqd_flags(krb5_kdc_req *req, krb5_db_entry *server,
                          krb5_ticket *tkt,
                          krb5_timestamp kdc_time, const char **status)
@@ -174,30 +174,30 @@ check_tgs_svc_reqd_flags(krb5_kdc_req *req, krb5_db_entry *server,
     if (server->attributes & KRB5_KDB_REQUIRES_HW_AUTH) {
         if (!(tkt->enc_part2->flags & TKT_FLG_HW_AUTH)) {
             *status = "NO HW PREAUTH";
-            return KRB_ERR_GENERIC;
+            return KRB5KRB_ERR_GENERIC;
         }
     }
     if (server->attributes & KRB5_KDB_REQUIRES_PRE_AUTH) {
         if (!(tkt->enc_part2->flags & TKT_FLG_PRE_AUTH)) {
             *status = "NO PREAUTH";
-            return KRB_ERR_GENERIC;
+            return KRB5KRB_ERR_GENERIC;
         }
     }
     return 0;
 }
 
-static int
+static krb5_error_code
 check_tgs_svc_time(krb5_kdc_req *req, krb5_db_entry *server, krb5_ticket *tkt,
                    krb5_timestamp kdc_time, const char **status)
 {
     if (server->expiration && ts_after(kdc_time, server->expiration)) {
         *status = "SERVICE EXPIRED";
-        return KDC_ERR_SERVICE_EXP;
+        return KRB5KDC_ERR_SERVICE_EXP;
     }
     return 0;
 }
 
-static int
+static krb5_error_code
 check_tgs_svc_policy(krb5_kdc_req *req, krb5_db_entry *server,
                      krb5_ticket *tkt, krb5_timestamp kdc_time,
                      const char **status)
@@ -217,7 +217,7 @@ check_tgs_svc_policy(krb5_kdc_req *req, krb5_db_entry *server,
 /*
  * Check header ticket timestamps against the current time.
  */
-static int
+static krb5_error_code
 check_tgs_times(krb5_kdc_req *req, krb5_ticket_times *times,
                 krb5_timestamp kdc_time, const char **status)
 {
@@ -229,7 +229,7 @@ check_tgs_times(krb5_kdc_req *req, krb5_ticket_times *times,
         starttime = times->starttime ? times->starttime : times->authtime;
         if (ts_after(starttime, kdc_time)) {
             *status = "NOT_YET_VALID";
-            return KRB_AP_ERR_TKT_NYV;
+            return KRB5KRB_AP_ERR_TKT_NYV;
         }
     }
     /*
@@ -239,46 +239,47 @@ check_tgs_times(krb5_kdc_req *req, krb5_ticket_times *times,
     if ((req->kdc_options & KDC_OPT_RENEW) &&
         ts_after(kdc_time, times->renew_till)) {
         *status = "TKT_EXPIRED";
-        return KRB_AP_ERR_TKT_EXPIRED;
+        return KRB5KRB_AP_ERR_TKT_EXPIRED;
     }
     return 0;
 }
 
 /* Check for local user tickets issued by foreign realms.  This check is
  * skipped for S4U2Self requests. */
-static int
+static krb5_error_code
 check_tgs_lineage(krb5_db_entry *server, krb5_ticket *tkt,
                   krb5_boolean is_crossrealm, const char **status)
 {
     if (is_crossrealm && data_eq(tkt->enc_part2->client->realm,
                                  server->princ->realm)) {
         *status = "INVALID LINEAGE";
-        return KDC_ERR_POLICY;
+        return KRB5KDC_ERR_POLICY;
     }
     return 0;
 }
 
-static int
-check_tgs_s4u2self(kdc_realm_t *kdc_active_realm, krb5_kdc_req *req,
+static krb5_error_code
+check_tgs_s4u2self(kdc_realm_t *realm, krb5_kdc_req *req,
                    krb5_db_entry *server, krb5_ticket *tkt, krb5_pac pac,
                    krb5_timestamp kdc_time,
                    krb5_pa_s4u_x509_user *s4u_x509_user, krb5_db_entry *client,
                    krb5_boolean is_crossrealm, krb5_boolean is_referral,
                    const char **status, krb5_pa_data ***e_data)
 {
+    krb5_context context = realm->realm_context;
     krb5_db_entry empty_server = { 0 };
 
     /* If the server is local, check that the request is for self. */
     if (!is_referral &&
-        !is_client_db_alias(kdc_context, server, tkt->enc_part2->client)) {
+        !is_client_db_alias(context, server, tkt->enc_part2->client)) {
         *status = "INVALID_S4U2SELF_REQUEST_SERVER_MISMATCH";
-        return KRB_AP_ERR_BADMATCH;
+        return KRB5KRB_AP_ERR_BADMATCH;
     }
 
     /* S4U2Self requests must use options valid for AS requests. */
     if (req->kdc_options & AS_INVALID_OPTIONS) {
         *status = "INVALID S4U2SELF OPTIONS";
-        return KDC_ERR_BADOPTION;
+        return KRB5KDC_ERR_BADOPTION;
     }
 
     /*
@@ -298,13 +299,13 @@ check_tgs_s4u2self(kdc_realm_t *kdc_active_realm, krb5_kdc_req *req,
         /* This could happen if the requesting server no longer exists, and we
          * found a referral instead.  Treat this as a server lookup failure. */
         *status = "LOOKING_UP_SERVER";
-        return KDC_ERR_S_PRINCIPAL_UNKNOWN;
+        return KRB5KDC_ERR_S_PRINCIPAL_UNKNOWN;
     }
     if (client != NULL && is_crossrealm && !is_referral) {
         /* A local server should not need a cross-realm TGT to impersonate
          * a local principal. */
         *status = "NOT_CROSS_REALM_REQUEST";
-        return KDC_ERR_C_PRINCIPAL_UNKNOWN; /* match Windows error */
+        return KRB5KDC_ERR_C_PRINCIPAL_UNKNOWN; /* match Windows error */
     }
     if (client == NULL && !is_crossrealm) {
         /*
@@ -313,7 +314,7 @@ check_tgs_s4u2self(kdc_realm_t *kdc_active_realm, krb5_kdc_req *req,
          * follow referrals back to us.
          */
         *status = "S4U2SELF_CLIENT_NOT_OURS";
-        return KDC_ERR_POLICY; /* match Windows error */
+        return KRB5KDC_ERR_POLICY; /* match Windows error */
     }
     if (client == NULL && s4u_x509_user->user_id.user->length == 0) {
         /*
@@ -322,35 +323,34 @@ check_tgs_s4u2self(kdc_realm_t *kdc_active_realm, krb5_kdc_req *req,
          * the subject-certificate field.
          */
         *status = "INVALID_XREALM_S4U2SELF_REQUEST";
-        return KDC_ERR_POLICY; /* match Windows error */
+        return KRB5KDC_ERR_POLICY; /* match Windows error */
     }
 
     /* The header ticket PAC must be present. */
     if (pac == NULL) {
         *status = "S4U2SELF_NO_PAC";
-        return KDC_ERR_TGT_REVOKED;
+        return KRB5KDC_ERR_TGT_REVOKED;
     }
 
     if (client != NULL) {
         /* The header ticket PAC must be for the impersonator. */
-        if (krb5_pac_verify(kdc_context, pac, tkt->enc_part2->times.authtime,
+        if (krb5_pac_verify(context, pac, tkt->enc_part2->times.authtime,
                             tkt->enc_part2->client, NULL, NULL) != 0) {
             *status = "S4U2SELF_LOCAL_PAC_CLIENT";
-            return KDC_ERR_BADOPTION;
+            return KRB5KDC_ERR_BADOPTION;
         }
 
         /* Validate the client policy.  Use an empty server principal to bypass
          * server policy checks. */
-        return validate_as_request(kdc_active_realm, req, client,
-                                   &empty_server, kdc_time, status, e_data);
+        return validate_as_request(realm, req, client, &empty_server, kdc_time,
+                                   status, e_data);
     } else {
         /* The header ticket PAC must be for the subject, with realm. */
-        if (krb5_pac_verify_ext(kdc_context, pac,
-                                tkt->enc_part2->times.authtime,
+        if (krb5_pac_verify_ext(context, pac, tkt->enc_part2->times.authtime,
                                 s4u_x509_user->user_id.user, NULL, NULL,
                                 TRUE) != 0) {
             *status = "S4U2SELF_FOREIGN_PAC_CLIENT";
-            return KDC_ERR_BADOPTION;
+            return KRB5KDC_ERR_BADOPTION;
         }
     }
 
@@ -420,8 +420,8 @@ cleanup:
     return result;
 }
 
-static int
-check_tgs_s4u2proxy(kdc_realm_t *kdc_active_realm, krb5_kdc_req *req,
+static krb5_error_code
+check_tgs_s4u2proxy(krb5_context context, krb5_kdc_req *req,
                     krb5_db_entry *server, krb5_ticket *tkt, krb5_pac pac,
                     const krb5_ticket *stkt, krb5_pac stkt_pac,
                     krb5_db_entry *stkt_server, krb5_boolean is_crossrealm,
@@ -430,35 +430,35 @@ check_tgs_s4u2proxy(kdc_realm_t *kdc_active_realm, krb5_kdc_req *req,
     /* A forwardable second ticket must be present in the request. */
     if (stkt == NULL) {
         *status = "NO_2ND_TKT";
-        return KDC_ERR_BADOPTION;
+        return KRB5KDC_ERR_BADOPTION;
     }
     if (!(stkt->enc_part2->flags & TKT_FLG_FORWARDABLE)) {
         *status = "EVIDENCE_TKT_NOT_FORWARDABLE";
-        return KDC_ERR_BADOPTION;
+        return KRB5KDC_ERR_BADOPTION;
     }
 
     /* Constrained delegation is mutually exclusive with renew/forward/etc.
      * (and therefore requires the header ticket to be a TGT). */
     if (req->kdc_options & (NON_TGT_OPTION | KDC_OPT_ENC_TKT_IN_SKEY)) {
         *status = "INVALID_S4U2PROXY_OPTIONS";
-        return KDC_ERR_BADOPTION;
+        return KRB5KDC_ERR_BADOPTION;
     }
 
     /* Can't get a TGT (otherwise it would be unconstrained delegation). */
     if (krb5_is_tgs_principal(req->server)) {
         *status = "NOT_ALLOWED_TO_DELEGATE";
-        return KDC_ERR_POLICY;
+        return KRB5KDC_ERR_POLICY;
     }
 
     /* The header ticket PAC must be present and for the impersonator. */
     if (pac == NULL) {
         *status = "S4U2PROXY_NO_HEADER_PAC";
-        return KDC_ERR_TGT_REVOKED;
+        return KRB5KDC_ERR_TGT_REVOKED;
     }
-    if (krb5_pac_verify(kdc_context, pac, tkt->enc_part2->times.authtime,
+    if (krb5_pac_verify(context, pac, tkt->enc_part2->times.authtime,
                         tkt->enc_part2->client, NULL, NULL) != 0) {
         *status = "S4U2PROXY_HEADER_PAC";
-        return KDC_ERR_BADOPTION;
+        return KRB5KDC_ERR_BADOPTION;
     }
 
     /*
@@ -470,24 +470,23 @@ check_tgs_s4u2proxy(kdc_realm_t *kdc_active_realm, krb5_kdc_req *req,
 
     if (stkt_pac == NULL) {
         *status = "S4U2PROXY_NO_STKT_PAC";
-        return KRB_AP_ERR_MODIFIED;
+        return KRB5KRB_AP_ERR_MODIFIED;
     }
     if (!is_crossrealm) {
         /* For an initial or same-realm request, the second ticket server and
          * header ticket client must be the same principal. */
-        if (!is_client_db_alias(kdc_context, stkt_server,
+        if (!is_client_db_alias(context, stkt_server,
                                 tkt->enc_part2->client)) {
             *status = "EVIDENCE_TICKET_MISMATCH";
-            return KDC_ERR_SERVER_NOMATCH;
+            return KRB5KDC_ERR_SERVER_NOMATCH;
         }
 
         /* The second ticket client and PAC client are the subject, and must
          * match. */
-        if (krb5_pac_verify(kdc_context, stkt_pac,
-                            stkt->enc_part2->times.authtime,
+        if (krb5_pac_verify(context, stkt_pac, stkt->enc_part2->times.authtime,
                             stkt->enc_part2->client, NULL, NULL) != 0) {
             *status = "S4U2PROXY_LOCAL_STKT_PAC";
-            return KDC_ERR_BADOPTION;
+            return KRB5KDC_ERR_BADOPTION;
         }
 
     } else {
@@ -499,54 +498,107 @@ check_tgs_s4u2proxy(kdc_realm_t *kdc_active_realm, krb5_kdc_req *req,
          */
         if (is_referral || !is_cross_tgs_principal(stkt_server->princ) ||
             !data_eq(stkt_server->princ->data[1], server->princ->realm) ||
-            !krb5_principal_compare(kdc_context, stkt->enc_part2->client,
+            !krb5_principal_compare(context, stkt->enc_part2->client,
                                     tkt->enc_part2->client)) {
             *status = "XREALM_EVIDENCE_TICKET_MISMATCH";
-            return KDC_ERR_BADOPTION;
+            return KRB5KDC_ERR_BADOPTION;
         }
 
         /* The second ticket PAC must be present and for the impersonated
          * client, with delegation info. */
         if (stkt_pac == NULL ||
-            verify_deleg_pac(kdc_context, stkt_pac, stkt->enc_part2,
+            verify_deleg_pac(context, stkt_pac, stkt->enc_part2,
                              req->server) != 0) {
             *status = "S4U2PROXY_CROSS_STKT_PAC";
-            return KDC_ERR_BADOPTION;
+            return KRB5KDC_ERR_BADOPTION;
         }
     }
 
     return 0;
 }
 
-static int
-check_tgs_u2u(kdc_realm_t *kdc_active_realm, krb5_kdc_req *req,
-              const krb5_ticket *stkt, krb5_db_entry *server,
-              const char **status)
+/* Check the KDB policy for a final RBCD request. */
+static krb5_error_code
+check_s4u2proxy_policy(krb5_context context, krb5_kdc_req *req,
+                       krb5_principal desired_client,
+                       krb5_principal impersonator_name,
+                       krb5_db_entry *impersonator, krb5_pac impersonator_pac,
+                       krb5_principal resource_name, krb5_db_entry *resource,
+                       krb5_boolean is_crossrealm, krb5_boolean is_referral,
+                       const char **status)
+{
+    krb5_error_code ret;
+    krb5_boolean support_rbcd, policy_denial = FALSE;
+
+    /* Check if the client supports resource-based constrained delegation. */
+    ret = kdc_get_pa_pac_rbcd(context, req->padata, &support_rbcd);
+    if (ret)
+        return ret;
+
+    if (is_referral) {
+        if (!support_rbcd) {
+            /* The client must support RBCD for a referral to be useful. */
+            *status = "UNSUPPORTED_S4U2PROXY_REQUEST";
+            return KRB5KDC_ERR_BADOPTION;
+        }
+        /* Policy will be checked in the resource realm. */
+        return 0;
+    }
+
+    /* Try resource-based authorization if the client supports RBCD. */
+    if (support_rbcd) {
+        ret = krb5_db_allowed_to_delegate_from(context, desired_client,
+                                               impersonator_name,
+                                               impersonator_pac, resource);
+        if (ret == KRB5KDC_ERR_BADOPTION)
+            policy_denial = TRUE;
+        else if (ret != KRB5_PLUGIN_OP_NOTSUPP)
+            return ret;
+    }
+
+    /* Try traditional authorization if the requestor is in this realm. */
+    if (!is_crossrealm) {
+        ret = krb5_db_check_allowed_to_delegate(context, desired_client,
+                                                impersonator, resource_name);
+        if (ret == KRB5KDC_ERR_BADOPTION)
+            policy_denial = TRUE;
+        else if (ret != KRB5_PLUGIN_OP_NOTSUPP)
+            return ret;
+    }
+
+    *status = policy_denial ? "NOT_ALLOWED_TO_DELEGATE" :
+        "UNSUPPORTED_S4U2PROXY_REQUEST";
+    return KRB5KDC_ERR_BADOPTION;
+}
+
+static krb5_error_code
+check_tgs_u2u(krb5_context context, krb5_kdc_req *req, const krb5_ticket *stkt,
+              krb5_db_entry *server, const char **status)
 {
     /* A second ticket must be present in the request. */
     if (stkt == NULL) {
         *status = "NO_2ND_TKT";
-        return KDC_ERR_BADOPTION;
+        return KRB5KDC_ERR_BADOPTION;
     }
 
     /* The second ticket must be a TGT to the server realm. */
     if (!is_local_tgs_principal(stkt->server) ||
         !data_eq(stkt->server->data[1], server->princ->realm)) {
         *status = "2ND_TKT_NOT_TGS";
-        return KDC_ERR_POLICY;
+        return KRB5KDC_ERR_POLICY;
     }
 
     /* The second ticket client must match the requested server. */
-    if (!is_client_db_alias(kdc_context, server, stkt->enc_part2->client)) {
+    if (!is_client_db_alias(context, server, stkt->enc_part2->client)) {
         *status = "2ND_TKT_MISMATCH";
-        return KDC_ERR_SERVER_NOMATCH;
+        return KRB5KDC_ERR_SERVER_NOMATCH;
     }
 
     return 0;
 }
 
 /* Validate the PAC of a non-S4U TGS request, if one is present. */
-static int
+static krb5_error_code
 check_normal_tgs_pac(krb5_context context, krb5_enc_tkt_part *enc_tkt,
                      krb5_pac pac, krb5_db_entry *server,
                      krb5_boolean is_crossrealm, const char **status)
@@ -568,7 +620,7 @@ check_normal_tgs_pac(krb5_context context, krb5_enc_tkt_part *enc_tkt,
         return 0;
 
     *status = "HEADER_PAC";
-    return KDC_ERR_BADOPTION;
+    return KRB5KDC_ERR_BADOPTION;
 }
 
 /*
@@ -576,19 +628,19 @@ check_normal_tgs_pac(krb5_context context, krb5_enc_tkt_part *enc_tkt,
  * checks that are peculiar to these cases.  (e.g., ticket service principal
  * matches requested service principal)
  */
-static int
-check_tgs_nontgt(kdc_realm_t *kdc_active_realm,
-                 krb5_kdc_req *req, krb5_ticket *tkt, const char **status)
+static krb5_error_code
+check_tgs_nontgt(krb5_context context, krb5_kdc_req *req, krb5_ticket *tkt,
+                 const char **status)
 {
-    if (!krb5_principal_compare(kdc_context, tkt->server, req->server)) {
+    if (!krb5_principal_compare(context, tkt->server, req->server)) {
         *status = "SERVER DIDN'T MATCH TICKET FOR RENEW/FORWARD/ETC";
-        return KDC_ERR_SERVER_NOMATCH;
+        return KRB5KDC_ERR_SERVER_NOMATCH;
     }
     /* Cannot proxy ticket granting tickets. */
     if ((req->kdc_options & KDC_OPT_PROXY) &&
         krb5_is_tgs_principal(req->server)) {
         *status = "CAN'T PROXY TGT";
-        return KDC_ERR_BADOPTION;
+        return KRB5KDC_ERR_BADOPTION;
     }
     return 0;
 }
@@ -597,37 +649,34 @@ check_tgs_nontgt(kdc_realm_t *kdc_active_realm,
  * Do some checks for a normal TGS-REQ (where the ticket service must be a TGS
  * principal).
  */
-static int
-check_tgs_tgt(kdc_realm_t *kdc_active_realm, krb5_kdc_req *req,
-              krb5_ticket *tkt, const char **status)
+static krb5_error_code
+check_tgs_tgt(krb5_kdc_req *req, krb5_ticket *tkt, const char **status)
 {
     /* Make sure it's a TGS principal. */
     if (!krb5_is_tgs_principal(tkt->server)) {
         *status = "BAD TGS SERVER NAME";
-        return KRB_AP_ERR_NOT_US;
+        return KRB5KRB_AP_ERR_NOT_US;
     }
     /* TGS principal second component must match service realm. */
-    if (!data_eq(*krb5_princ_component(kdc_context, tkt->server, 1),
-                 *krb5_princ_realm(kdc_context, req->server))) {
+    if (!data_eq(tkt->server->data[1], req->server->realm)) {
         *status = "BAD TGS SERVER INSTANCE";
-        return KRB_AP_ERR_NOT_US;
+        return KRB5KRB_AP_ERR_NOT_US;
     }
     return 0;
 }
 
-int
-validate_tgs_request(kdc_realm_t *kdc_active_realm,
-                     krb5_kdc_req *request, krb5_db_entry *server,
-                     krb5_ticket *ticket, krb5_pac pac,
-                     const krb5_ticket *stkt, krb5_pac stkt_pac,
-                     krb5_db_entry *stkt_server, krb5_timestamp kdc_time,
-                     krb5_pa_s4u_x509_user *s4u_x509_user,
-                     krb5_db_entry *s4u2self_client,
-                     krb5_boolean is_crossrealm, krb5_boolean is_referral,
-                     const char **status, krb5_pa_data ***e_data)
+krb5_error_code
+check_tgs_constraints(kdc_realm_t *realm, krb5_kdc_req *request,
+                      krb5_db_entry *server, krb5_ticket *ticket, krb5_pac pac,
+                      const krb5_ticket *stkt, krb5_pac stkt_pac,
+                      krb5_db_entry *stkt_server, krb5_timestamp kdc_time,
+                      krb5_pa_s4u_x509_user *s4u_x509_user,
+                      krb5_db_entry *s4u2self_client,
+                      krb5_boolean is_crossrealm, krb5_boolean is_referral,
+                      const char **status, krb5_pa_data ***e_data)
 {
+    krb5_context context = realm->realm_context;
     int errcode;
-    krb5_error_code ret;
 
     /* Depends only on request and ticket. */
     errcode = check_tgs_opts(request, ticket, status);
@@ -640,28 +689,18 @@ validate_tgs_request(kdc_realm_t *kdc_active_realm,
     if (errcode != 0)
         return errcode;
 
-    errcode = check_tgs_svc_policy(request, server, ticket, kdc_time, status);
-    if (errcode != 0)
-        return errcode;
-
     if (request->kdc_options & NON_TGT_OPTION)
-        errcode = check_tgs_nontgt(kdc_active_realm, request, ticket, status);
+        errcode = check_tgs_nontgt(context, request, ticket, status);
     else
-        errcode = check_tgs_tgt(kdc_active_realm, request, ticket, status);
+        errcode = check_tgs_tgt(request, ticket, status);
     if (errcode != 0)
         return errcode;
 
-    /* Check the hot list */
-    if (check_hot_list(ticket)) {
-        *status = "HOT_LIST";
-        return(KRB_AP_ERR_REPEAT);
-    }
-
     if (s4u_x509_user != NULL) {
-        errcode = check_tgs_s4u2self(kdc_active_realm, request, server, ticket,
-                                     pac, kdc_time, s4u_x509_user,
-                                     s4u2self_client, is_crossrealm,
-                                     is_referral, status, e_data);
+        errcode = check_tgs_s4u2self(realm, request, server, ticket, pac,
+                                     kdc_time, s4u_x509_user, s4u2self_client,
+                                     is_crossrealm, is_referral, status,
+                                     e_data);
     } else {
         errcode = check_tgs_lineage(server, ticket, is_crossrealm, status);
     }
@@ -669,36 +708,64 @@ validate_tgs_request(kdc_realm_t *kdc_active_realm,
         return errcode;
 
     if (request->kdc_options & KDC_OPT_ENC_TKT_IN_SKEY) {
-        errcode = check_tgs_u2u(kdc_active_realm, request, stkt, server,
-                                status);
+        errcode = check_tgs_u2u(context, request, stkt, server, status);
         if (errcode != 0)
             return errcode;
     }
 
     if (request->kdc_options & KDC_OPT_CNAME_IN_ADDL_TKT) {
-        errcode = check_tgs_s4u2proxy(kdc_active_realm, request, server,
-                                      ticket, pac, stkt, stkt_pac, stkt_server,
+        errcode = check_tgs_s4u2proxy(context, request, server, ticket, pac,
+                                      stkt, stkt_pac, stkt_server,
                                       is_crossrealm, is_referral, status);
         if (errcode != 0)
             return errcode;
     } else if (s4u_x509_user == NULL) {
-        errcode = check_normal_tgs_pac(kdc_context, ticket->enc_part2, pac,
-                                       server, is_crossrealm, status);
+        errcode = check_normal_tgs_pac(context, ticket->enc_part2, pac, server,
+                                       is_crossrealm, status);
         if (errcode != 0)
             return errcode;
     }
 
-    if (check_anon(kdc_active_realm, ticket->enc_part2->client,
-                   request->server) != 0) {
+    return 0;
+}
+
+krb5_error_code
+check_tgs_policy(kdc_realm_t *realm, krb5_kdc_req *request,
+                 krb5_db_entry *server, krb5_ticket *ticket,
+                 krb5_pac pac, const krb5_ticket *stkt, krb5_pac stkt_pac,
+                 krb5_principal stkt_pac_client, krb5_db_entry *stkt_server,
+                 krb5_timestamp kdc_time, krb5_boolean is_crossrealm,
+                 krb5_boolean is_referral, const char **status,
+                 krb5_pa_data ***e_data)
+{
+    krb5_context context = realm->realm_context;
+    int errcode;
+    krb5_error_code ret;
+    krb5_principal desired_client;
+
+    errcode = check_tgs_svc_policy(request, server, ticket, kdc_time, status);
+    if (errcode != 0)
+        return errcode;
+
+    if (request->kdc_options & KDC_OPT_CNAME_IN_ADDL_TKT) {
+        desired_client = (stkt_pac_client != NULL) ? stkt_pac_client :
+            stkt->enc_part2->client;
+        errcode = check_s4u2proxy_policy(context, request, desired_client,
+                                         ticket->enc_part2->client,
+                                         stkt_server, pac, request->server,
+                                         server, is_crossrealm, is_referral,
+                                         status);
+        if (errcode != 0)
+            return errcode;
+    }
+
+    if (check_anon(realm, ticket->enc_part2->client, request->server) != 0) {
         *status = "ANONYMOUS NOT ALLOWED";
-        return(KDC_ERR_POLICY);
+        return KRB5KDC_ERR_POLICY;
     }
 
     /* Perform KDB module policy checks. */
-    ret = krb5_db_check_policy_tgs(kdc_context, request, server, ticket,
-                                   status, e_data);
-    if (ret && ret != KRB5_PLUGIN_OP_NOTSUPP)
-        return errcode_to_protocol(ret);
-
-    return 0;
+    ret = krb5_db_check_policy_tgs(context, request, server, ticket, status,
+                                   e_data);
+    return (ret == KRB5_PLUGIN_OP_NOTSUPP) ? 0 : ret;
 }
