@@ -35,14 +35,12 @@ krb5_c_random_seed(krb5_context context, krb5_data *data)
 /* Routines to get entropy from the OS. */
 #if defined(_WIN32)
 
-krb5_boolean
-k5_get_os_entropy(unsigned char *buf, size_t len, int strong)
+static krb5_boolean
+get_os_entropy(unsigned char *buf, size_t len)
 {
     krb5_boolean result;
     HCRYPTPROV provider;
 
-    /* CryptGenRandom is always considered strong. */
-
     if (!CryptAcquireContext(&provider, NULL, NULL, PROV_RSA_FULL,
                              CRYPT_VERIFYCONTEXT))
         return FALSE;
@@ -95,10 +93,9 @@ cleanup:
     return result;
 }
 
-krb5_boolean
-k5_get_os_entropy(unsigned char *buf, size_t len, int strong)
+static krb5_boolean
+get_os_entropy(unsigned char *buf, size_t len)
 {
-    const char *device;
 #if defined(__linux__) && defined(SYS_getrandom)
     int r;
 
@@ -127,8 +124,30 @@ k5_get_os_entropy(unsigned char *buf, size_t len, int strong)
         return TRUE;
 #endif /* defined(__linux__) && defined(SYS_getrandom) */
 
-    device = strong ? "/dev/random" : "/dev/urandom";
-    return read_entropy_from_device(device, buf, len);
+    return read_entropy_from_device("/dev/urandom", buf, len);
 }
 
 #endif /* not Windows */
+
+krb5_error_code KRB5_CALLCONV
+krb5_c_random_make_octets(krb5_context context, krb5_data *outdata)
+{
+    krb5_boolean res;
+
+    res = get_os_entropy((uint8_t *)outdata->data, outdata->length);
+    return res ? 0 : KRB5_CRYPTO_INTERNAL;
+}
+
+krb5_error_code KRB5_CALLCONV
+krb5_c_random_add_entropy(krb5_context context, unsigned int randsource,
+                          const krb5_data *indata)
+{
+    return 0;
+}
+
+krb5_error_code KRB5_CALLCONV
+krb5_c_random_os_entropy(krb5_context context, int strong, int *success)
+{
+    *success = 0;
+    return 0;
+}
