@@ -1370,7 +1370,7 @@ get_ldap_auth_ind(krb5_context context, LDAP *ld, LDAPMessage *ldap_ent,
 {
     krb5_error_code ret;
     int i;
-    char **auth_inds = NULL;
+    char **auth_inds = NULL, *indstr;
     struct k5buf buf = EMPTY_K5BUF;
 
     auth_inds = ldap_get_values(ld, ldap_ent, "krbPrincipalAuthInd");
@@ -1386,12 +1386,14 @@ get_ldap_auth_ind(krb5_context context, LDAP *ld, LDAPMessage *ldap_ent,
             k5_buf_add(&buf, " ");
     }
 
-    ret = k5_buf_status(&buf);
-    if (ret)
+    indstr = k5_buf_cstring(&buf);
+    if (indstr == NULL) {
+        ret = ENOMEM;
         goto cleanup;
+    }
 
     ret = krb5_dbe_set_string(context, entry, KRB5_KDB_SK_REQUIRE_AUTH,
-                              buf.data);
+                              indstr);
     if (!ret)
         *mask |= KDB_AUTH_IND_ATTR;
 
