@@ -507,7 +507,7 @@ ktest_make_sample_sam_challenge_2_body(krb5_sam_challenge_2_body *p)
     krb5_data_parse(&p->sam_response_prompt, "response_prompt ipse");
     p->sam_pk_for_sad = empty_data();
     p->sam_nonce = 0x543210;
-    p->sam_etype = ENCTYPE_DES_CBC_CRC;
+    p->sam_etype = ENCTYPE_AES256_CTS_HMAC_SHA384_192;
 }
 
 void
@@ -518,7 +518,7 @@ ktest_make_sample_sam_response_2(krb5_sam_response_2 *p)
     p->sam_flags = KRB5_SAM_USE_SAD_AS_KEY; /* KRB5_SAM_* values */
     krb5_data_parse(&p->sam_track_id, "track data");
     krb5_data_parse(&p->sam_enc_nonce_or_sad.ciphertext, "nonce or sad");
-    p->sam_enc_nonce_or_sad.enctype = ENCTYPE_DES_CBC_CRC;
+    p->sam_enc_nonce_or_sad.enctype = ENCTYPE_AES256_CTS_HMAC_SHA384_192;
     p->sam_enc_nonce_or_sad.kvno = 3382;
     p->sam_nonce = 0x543210;
 }
@@ -559,28 +559,6 @@ ktest_make_sample_ad_kdcissued(krb5_ad_kdcissued *p)
     ktest_make_sample_authorization_data(&p->elements);
 }
 
-void
-ktest_make_sample_ad_signedpath_data(krb5_ad_signedpath_data *p)
-{
-    ktest_make_sample_principal(&p->client);
-    p->authtime = SAMPLE_TIME;
-    p->delegated = ealloc(2 * sizeof(krb5_principal));
-    ktest_make_sample_principal(&p->delegated[0]);
-    p->delegated[1] = NULL;
-    ktest_make_sample_authorization_data(&p->authorization_data);
-    ktest_make_sample_pa_data_array(&p->method_data);
-}
-
-void
-ktest_make_sample_ad_signedpath(krb5_ad_signedpath *p)
-{
-    p->enctype = 1;
-    ktest_make_sample_checksum(&p->checksum);
-    p->delegated = ealloc(2 * sizeof(krb5_principal));
-    p->delegated[1] = NULL;
-    ktest_make_sample_pa_data_array(&p->method_data);
-}
-
 void
 ktest_make_sample_iakerb_header(krb5_iakerb_header *ih)
 {
@@ -729,15 +707,6 @@ ktest_make_sample_pk_authenticator(krb5_pk_authenticator *p)
     ktest_make_sample_data(p->freshnessToken);
 }
 
-static void
-ktest_make_sample_pk_authenticator_draft9(krb5_pk_authenticator_draft9 *p)
-{
-    ktest_make_sample_principal(&p->kdcName);
-    p->cusec = SAMPLE_USEC;
-    p->ctime = SAMPLE_TIME;
-    p->nonce = SAMPLE_NONCE;
-}
-
 static void
 ktest_make_sample_oid(krb5_data *p)
 {
@@ -759,13 +728,6 @@ ktest_make_sample_algorithm_identifier_no_params(krb5_algorithm_identifier *p)
     p->parameters = empty_data();
 }
 
-static void
-ktest_make_sample_subject_pk_info(krb5_subject_pk_info *p)
-{
-    ktest_make_sample_algorithm_identifier(&p->algorithm);
-    ktest_make_sample_data(&p->subjectPublicKey);
-}
-
 static void
 ktest_make_sample_external_principal_identifier(
     krb5_external_principal_identifier *p)
@@ -788,13 +750,6 @@ ktest_make_sample_pa_pk_as_req(krb5_pa_pk_as_req *p)
     ktest_make_sample_data(&p->kdcPkId);
 }
 
-void
-ktest_make_sample_pa_pk_as_req_draft9(krb5_pa_pk_as_req_draft9 *p)
-{
-    ktest_make_sample_data(&p->signedAuthPack);
-    ktest_make_sample_data(&p->kdcCert);
-}
-
 static void
 ktest_make_sample_dh_rep_info(krb5_dh_rep_info *p)
 {
@@ -818,26 +773,12 @@ ktest_make_sample_pa_pk_as_rep_encKeyPack(krb5_pa_pk_as_rep *p)
     ktest_make_sample_data(&p->u.encKeyPack);
 }
 
-void
-ktest_make_sample_pa_pk_as_rep_draft9_dhSignedData(krb5_pa_pk_as_rep_draft9 *p)
-{
-    p->choice = choice_pa_pk_as_rep_draft9_dhSignedData;
-    ktest_make_sample_data(&p->u.dhSignedData);
-}
-
-void
-ktest_make_sample_pa_pk_as_rep_draft9_encKeyPack(krb5_pa_pk_as_rep_draft9 *p)
-{
-    p->choice = choice_pa_pk_as_rep_draft9_encKeyPack;
-    ktest_make_sample_data(&p->u.encKeyPack);
-}
-
 void
 ktest_make_sample_auth_pack(krb5_auth_pack *p)
 {
     ktest_make_sample_pk_authenticator(&p->pkAuthenticator);
-    p->clientPublicValue = ealloc(sizeof(krb5_subject_pk_info));
-    ktest_make_sample_subject_pk_info(p->clientPublicValue);
+    /* Need a valid DER encoding here; this is the OCTET STRING "pvalue". */
+    krb5_data_parse(&p->clientPublicValue, "\x04\x06" "pvalue");
     p->supportedCMSTypes = ealloc(3 * sizeof(krb5_algorithm_identifier *));
     p->supportedCMSTypes[0] = ealloc(sizeof(krb5_algorithm_identifier));
     ktest_make_sample_algorithm_identifier(p->supportedCMSTypes[0]);
@@ -851,14 +792,6 @@ ktest_make_sample_auth_pack(krb5_auth_pack *p)
     p->supportedKDFs[1] = NULL;
 }
 
-void
-ktest_make_sample_auth_pack_draft9(krb5_auth_pack_draft9 *p)
-{
-    ktest_make_sample_pk_authenticator_draft9(&p->pkAuthenticator);
-    p->clientPublicValue = ealloc(sizeof(krb5_subject_pk_info));
-    ktest_make_sample_subject_pk_info(p->clientPublicValue);
-}
-
 void
 ktest_make_sample_kdc_dh_key_info(krb5_kdc_dh_key_info *p)
 {
@@ -874,13 +807,6 @@ ktest_make_sample_reply_key_pack(krb5_reply_key_pack *p)
     ktest_make_sample_checksum(&p->asChecksum);
 }
 
-void
-ktest_make_sample_reply_key_pack_draft9(krb5_reply_key_pack_draft9 *p)
-{
-    ktest_make_sample_keyblock(&p->replyKey);
-    p->nonce = SAMPLE_NONCE;
-}
-
 void
 ktest_make_sample_sp80056a_other_info(krb5_sp80056a_other_info *p)
 {
@@ -893,7 +819,7 @@ ktest_make_sample_sp80056a_other_info(krb5_sp80056a_other_info *p)
 void
 ktest_make_sample_pkinit_supp_pub_info(krb5_pkinit_supp_pub_info *p)
 {
-    p->enctype = ENCTYPE_DES_CBC_CRC;
+    p->enctype = ENCTYPE_AES256_CTS_HMAC_SHA384_192;
     ktest_make_sample_data(&p->as_req);
     ktest_make_sample_data(&p->pk_as_rep);
 }
@@ -1576,39 +1502,6 @@ ktest_empty_ad_kdcissued(krb5_ad_kdcissued *p)
     ktest_destroy_authorization_data(&p->elements);
 }
 
-void
-ktest_empty_ad_signedpath_data(krb5_ad_signedpath_data *p)
-{
-    int i;
-
-    ktest_destroy_principal(&p->client);
-    if (p->delegated != NULL) {
-        for (i = 0; p->delegated[i] != NULL; i++) {
-            krb5_principal princ = p->delegated[i];
-            ktest_destroy_principal(&princ);
-        }
-        free(p->delegated);
-    }
-    ktest_destroy_pa_data_array(&p->method_data);
-    ktest_destroy_authorization_data(&p->authorization_data);
-}
-
-void
-ktest_empty_ad_signedpath(krb5_ad_signedpath *p)
-{
-    int i;
-
-    free(p->checksum.contents);
-    if (p->delegated != NULL) {
-        for (i = 0; p->delegated[i] != NULL; i++) {
-            krb5_principal princ = p->delegated[i];
-            ktest_destroy_principal(&princ);
-        }
-        free(p->delegated);
-    }
-    ktest_destroy_pa_data_array(&p->method_data);
-}
-
 void
 ktest_empty_iakerb_header(krb5_iakerb_header *p)
 {
@@ -1717,19 +1610,6 @@ ktest_empty_pk_authenticator(krb5_pk_authenticator *p)
     p->freshnessToken = NULL;
 }
 
-static void
-ktest_empty_pk_authenticator_draft9(krb5_pk_authenticator_draft9 *p)
-{
-    ktest_destroy_principal(&p->kdcName);
-}
-
-static void
-ktest_empty_subject_pk_info(krb5_subject_pk_info *p)
-{
-    ktest_empty_algorithm_identifier(&p->algorithm);
-    ktest_empty_data(&p->subjectPublicKey);
-}
-
 static void
 ktest_empty_external_principal_identifier(
     krb5_external_principal_identifier *p)
@@ -1754,13 +1634,6 @@ ktest_empty_pa_pk_as_req(krb5_pa_pk_as_req *p)
     ktest_empty_data(&p->kdcPkId);
 }
 
-void
-ktest_empty_pa_pk_as_req_draft9(krb5_pa_pk_as_req_draft9 *p)
-{
-    ktest_empty_data(&p->signedAuthPack);
-    ktest_empty_data(&p->kdcCert);
-}
-
 static void
 ktest_empty_dh_rep_info(krb5_dh_rep_info *p)
 {
@@ -1779,16 +1652,6 @@ ktest_empty_pa_pk_as_rep(krb5_pa_pk_as_rep *p)
     p->choice = choice_pa_pk_as_rep_UNKNOWN;
 }
 
-void
-ktest_empty_pa_pk_as_rep_draft9(krb5_pa_pk_as_rep_draft9 *p)
-{
-    if (p->choice == choice_pa_pk_as_rep_draft9_dhSignedData)
-        ktest_empty_data(&p->u.dhSignedData);
-    else if (p->choice == choice_pa_pk_as_rep_draft9_encKeyPack)
-        ktest_empty_data(&p->u.encKeyPack);
-    p->choice = choice_pa_pk_as_rep_draft9_UNKNOWN;
-}
-
 void
 ktest_empty_auth_pack(krb5_auth_pack *p)
 {
@@ -1796,11 +1659,7 @@ ktest_empty_auth_pack(krb5_auth_pack *p)
     krb5_data **d;
 
     ktest_empty_pk_authenticator(&p->pkAuthenticator);
-    if (p->clientPublicValue != NULL) {
-        ktest_empty_subject_pk_info(p->clientPublicValue);
-        free(p->clientPublicValue);
-        p->clientPublicValue = NULL;
-    }
+    ktest_empty_data(&p->clientPublicValue);
     if (p->supportedCMSTypes != NULL) {
         for (ai = p->supportedCMSTypes; *ai != NULL; ai++) {
             ktest_empty_algorithm_identifier(*ai);
@@ -1820,17 +1679,6 @@ ktest_empty_auth_pack(krb5_auth_pack *p)
     }
 }
 
-void
-ktest_empty_auth_pack_draft9(krb5_auth_pack_draft9 *p)
-{
-    ktest_empty_pk_authenticator_draft9(&p->pkAuthenticator);
-    if (p->clientPublicValue != NULL) {
-        ktest_empty_subject_pk_info(p->clientPublicValue);
-        free(p->clientPublicValue);
-        p->clientPublicValue = NULL;
-    }
-}
-
 void
 ktest_empty_kdc_dh_key_info(krb5_kdc_dh_key_info *p)
 {
@@ -1844,12 +1692,6 @@ ktest_empty_reply_key_pack(krb5_reply_key_pack *p)
     ktest_empty_checksum(&p->asChecksum);
 }
 
-void
-ktest_empty_reply_key_pack_draft9(krb5_reply_key_pack_draft9 *p)
-{
-    ktest_empty_keyblock(&p->replyKey);
-}
-
 void ktest_empty_sp80056a_other_info(krb5_sp80056a_other_info *p)
 {
     ktest_empty_algorithm_identifier(&p->algorithm_identifier);
